# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: 
#     Wenyang Zhou <576825820@qq.com>
#     Dun Liang <randonlang@gmail.com>. 
# 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
# This model is generated by pytorch converter.
import jittor as jt
from jittor import nn

__all__ = ['GoogLeNet', 'googlenet']

def googlenet(pretrained=False, **kwargs):
    model = GoogLeNet(**kwargs)
    if pretrained: model.load("jittorhub://googlenet.pkl")
    return model

class GoogLeNet(nn.Module):
    """ GoogLeNet model architecture.

    Args:

    * num_classes: Number of classes. Default: 1000.
    * aux_logits: If True, add an auxiliary branch that can improve training. Default: True
    * init_weights: Defualt: True.
    * blocks: List of three blocks, [conv_block, inception_block, inception_aux_block]. If None, will use [BasicConv2d, Inception, InceptionAux] instead. Default: None.
    """

    def __init__(self, num_classes=1000, aux_logits=True, init_weights=True, blocks=None):
        super(GoogLeNet, self).__init__()
        if (blocks is None):
            blocks = [BasicConv2d, Inception, InceptionAux]
        assert (len(blocks) == 3)
        conv_block = blocks[0]
        inception_block = blocks[1]
        inception_aux_block = blocks[2]
        self.aux_logits = aux_logits
        self.conv1 = conv_block(3, 64, kernel_size=7, stride=2, padding=3)
        self.maxpool1 = nn.Pool(3, stride=2, ceil_mode=True, op='maximum')
        self.conv2 = conv_block(64, 64, kernel_size=1)
        self.conv3 = conv_block(64, 192, kernel_size=3, padding=1)
        self.maxpool2 = nn.Pool(3, stride=2, ceil_mode=True, op='maximum')
        self.inception3a = inception_block(192, 64, 96, 128, 16, 32, 32)
        self.inception3b = inception_block(256, 128, 128, 192, 32, 96, 64)
        self.maxpool3 = nn.Pool(3, stride=2, ceil_mode=True, op='maximum')
        self.inception4a = inception_block(480, 192, 96, 208, 16, 48, 64)
        self.inception4b = inception_block(512, 160, 112, 224, 24, 64, 64)
        self.inception4c = inception_block(512, 128, 128, 256, 24, 64, 64)
        self.inception4d = inception_block(512, 112, 144, 288, 32, 64, 64)
        self.inception4e = inception_block(528, 256, 160, 320, 32, 128, 128)
        self.maxpool4 = nn.Pool(2, stride=2, ceil_mode=True, op='maximum')
        self.inception5a = inception_block(832, 256, 160, 320, 32, 128, 128)
        self.inception5b = inception_block(832, 384, 192, 384, 48, 128, 128)
        if aux_logits:
            self.aux1 = inception_aux_block(512, num_classes)
            self.aux2 = inception_aux_block(528, num_classes)
        else:
            self.aux1 = None
            self.aux2 = None
        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
        self.dropout = nn.Dropout(0.2)
        self.fc = nn.Linear(1024, num_classes)

    def _forward(self, x):
        x = self.conv1(x)
        x = self.maxpool1(x)
        x = self.conv2(x)
        x = self.conv3(x)
        x = self.maxpool2(x)
        x = self.inception3a(x)
        x = self.inception3b(x)
        x = self.maxpool3(x)
        x = self.inception4a(x)
        if (self.aux1 is not None):
            aux1 = self.aux1(x)
        x = self.inception4b(x)
        x = self.inception4c(x)
        x = self.inception4d(x)
        if (self.aux2 is not None):
            aux2 = self.aux2(x)
        x = self.inception4e(x)
        x = self.maxpool4(x)
        x = self.inception5a(x)
        x = self.inception5b(x)
        x = self.avgpool(x)

        x = jt.reshape(x, (x.shape[0], (- 1)))
        x = self.dropout(x)
        x = self.fc(x)
        return (x, aux2, aux1)

    def eager_outputs(self, x, aux2, aux1):
        return x

    def execute(self, x):
        (x, aux1, aux2) = self._forward(x)
        aux_defined = (self.aux_logits)
        return self.eager_outputs(x, aux2, aux1)

class Inception(nn.Module):

    def __init__(self, in_channels, ch1x1, ch3x3red, ch3x3, ch5x5red, ch5x5, pool_proj, conv_block=None):
        super(Inception, self).__init__()
        if (conv_block is None):
            conv_block = BasicConv2d
        self.branch1 = conv_block(in_channels, ch1x1, kernel_size=1)
        self.branch2 = nn.Sequential(conv_block(in_channels, ch3x3red, kernel_size=1), conv_block(ch3x3red, ch3x3, kernel_size=3, padding=1))
        self.branch3 = nn.Sequential(conv_block(in_channels, ch5x5red, kernel_size=1), conv_block(ch5x5red, ch5x5, kernel_size=3, padding=1))
        self.branch4 = nn.Sequential(nn.Pool(kernel_size=3, stride=1, padding=1, ceil_mode=True, op='maximum'), conv_block(in_channels, pool_proj, kernel_size=1))

    def _forward(self, x):
        branch1 = self.branch1(x)
        branch2 = self.branch2(x)
        branch3 = self.branch3(x)
        branch4 = self.branch4(x)
        outputs = [branch1, branch2, branch3, branch4]
        return outputs

    def execute(self, x):
        outputs = self._forward(x)
        return jt.contrib.concat(outputs, dim=1)

class InceptionAux(nn.Module):

    def __init__(self, in_channels, num_classes, conv_block=None):
        super(InceptionAux, self).__init__()
        if (conv_block is None):
            conv_block = BasicConv2d
        self.conv = conv_block(in_channels, 128, kernel_size=1)
        self.fc1 = nn.Linear(2048, 1024)
        self.fc2 = nn.Linear(1024, num_classes)

    def execute(self, x):
        x = nn.AdaptiveAvgPool2d(4)(x)
        x = self.conv(x)
        x = jt.reshape(x, (x.shape[0], (- 1)))
        x = nn.relu(self.fc1(x))
        x = nn.Dropout(0.7)(x)
        x = self.fc2(x)
        return x

class BasicConv2d(nn.Module):

    def __init__(self, in_channels, out_channels, **kwargs):
        super(BasicConv2d, self).__init__()
        self.conv = nn.Conv(in_channels, out_channels, bias=False, **kwargs)
        self.bn = nn.BatchNorm(out_channels, eps=0.001)

    def execute(self, x):
        x = self.conv(x)
        x = self.bn(x)
        return nn.relu(x)



# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: 
#     Dun Liang <randonlang@gmail.com>. 
# 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import unittest
import jittor as jt
import os
import numpy as np

from jittor.test.test_log import find_log_with_re
skip_this_test = False

try:
    jt.dirty_fix_pytorch_runtime_error()
    import torch
except:
    skip_this_test = True

@unittest.skipIf(skip_this_test, "No Torch found")
class TestConvTranspose(unittest.TestCase):

    @unittest.skipIf(not jt.has_cuda, "No CUDA found")
    @jt.flag_scope(use_cuda=1)
    def test_cuda(self):
        self.test()

    def test(self):
        def check(data_shape, weights_shape, stride=1, dilation=1, groups=1):
            N,C,H,W = data_shape
            i,o,h,w = weights_shape
            img = np.random.rand(N,C,H,W).astype("float32")
            weights = np.random.rand(i,o//groups,h,w).astype("float32")
            m1 = jt.nn.ConvTranspose(i,o,h, stride=stride, dilation=dilation, bias=False, groups=groups)
            m2 = torch.nn.ConvTranspose2d(i,o,h, stride=stride, dilation=dilation, bias=False, groups=groups)
            m1.weight.data = weights
            m2.weight.data = torch.Tensor(weights)
            x = jt.array(img)
            out1 = m1(x)
            mask = jt.random(out1.shape)
            out1 = out1*mask
            tx = torch.Tensor(img)
            tx.requires_grad = True
            out2 = m2(tx) * torch.Tensor(mask.data)
            with jt.log_capture_scope(log_silent=1, 
                log_vprefix="var_re=0,conv=0,op.cc=100") as logs:
                assert np.allclose(out1.data, out2.data)
                dx, dw = jt.grad(out1, [x, m1.weight])
                jt.sync([dx, dw])
                out2.sum().backward()
                assert np.allclose(dw.data, m2.weight.grad.numpy(), 1e-3)
                assert np.allclose(dx.data, tx.grad.numpy())
            assert len(find_log_with_re(logs, "conv")) == 3
        check((4, 5, 10, 10), (5, 6, 3, 3))
        check((4, 5, 10, 10), (5, 6, 3, 3), 2)
        check((4, 5, 100, 100), (5, 6, 4, 4), 2)
        check((4, 5, 100, 100), (5, 6, 4, 4), 3)
        check((4, 5, 100, 100), (5, 6, 5, 5), 1, 2)
        check((4, 5, 100, 100), (5, 6, 5, 5), 2, 2)
        check((4, 5, 100, 100), (5, 6, 5, 5), 2, 3)
        check((4, 6, 100, 100), (6, 6, 5, 5), 2, 3, 2)

    def test_function(self):
        def check(data_shape, weights_shape, stride=1, dilation=1):
            N,C,H,W = data_shape
            i,o,h,w = weights_shape
            img = np.random.rand(N,C,H,W).astype("float32")
            weights = np.random.rand(i,o,h,w).astype("float32")
            m1 = jt.nn.ConvTranspose(i,o,h, stride=stride, dilation=dilation, bias=False)
            m2 = torch.nn.ConvTranspose2d(i,o,h, stride=stride, dilation=dilation, bias=False)
            m1.weight.data = weights
            m2.weight.data = torch.Tensor(weights)
            x = jt.array(img)
            # out1 = m1(x)
            out1 = jt.nn.conv_transpose2d(x, m1.weight, stride=stride, dilation=dilation, bias=False)
            mask = jt.random(out1.shape)
            out1 = out1*mask
            tx = torch.Tensor(img)
            tx.requires_grad = True
            out2 = m2(tx) * torch.Tensor(mask.data)
            with jt.log_capture_scope(log_silent=1, 
                log_vprefix="var_re=0,conv=0,op.cc=100") as logs:
                assert np.allclose(out1.data, out2.data)
                dx, dw = jt.grad(out1, [x, m1.weight])
                jt.sync([dx, dw])
                out2.sum().backward()
                assert np.allclose(dw.data, m2.weight.grad.numpy(), 1e-3)
                assert np.allclose(dx.data, tx.grad.numpy())
            assert len(find_log_with_re(logs, "conv")) == 3
        check((4, 5, 10, 10), (5, 6, 3, 3))
        check((4, 5, 10, 10), (5, 6, 3, 3), 2)
        check((4, 5, 100, 100), (5, 6, 4, 4), 2)
        check((4, 5, 100, 100), (5, 6, 4, 4), 3)
        check((4, 5, 100, 100), (5, 6, 5, 5), 1, 2)
        check((4, 5, 100, 100), (5, 6, 5, 5), 2, 2)
        check((4, 5, 100, 100), (5, 6, 5, 5), 2, 3)

    def test_conv1d(self):
        conv1d = jt.nn.Conv1d(10,20,5)
        a = jt.rand((3,10,15))
        b = conv1d(a)
        b.sync()
        assert b.shape == [3,20,11]
        b = jt.nn.Conv1d(10,20,5, padding=2)(a)
        assert b.shape == [3,20,15]
        assert sorted(list(conv1d.state_dict().keys())) == ['bias', 'weight'], conv1d.state_dict().keys()
        
if __name__ == "__main__":
    unittest.main()



# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: Dun Liang <randonlang@gmail.com>. 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import unittest
import jittor as jt
import numpy as np
import os

def transpose0231(x):
    s0, s1, s2, s3 = x.shape
    asize = 16
    bsize = 16
    ILP = 2
    return jt.code([s0, s2, s3, s1], x.dtype, [x],
    cuda_header="#include <type/fp16_compute.h>\n#include <cassert>",
    cuda_src=f"""
    __global__ void kernel(in0_type* __restrict__ x, in0_type* __restrict__ y, int s0, int s1, int s2, int s3) {{
        __shared__ in0_type t[{asize*ILP}*{bsize*ILP+1}];
        int t3 = threadIdx.x % {bsize};
        int t1 = threadIdx.x / {bsize};
        int b3 = blockIdx.x;
        int b2 = blockIdx.y;
        int b0 = blockIdx.z;
        int x3 = 1;
        int x2 = s3;
        int x1 = s2*x2;
        int x0 = s1*x1;
        int y3 = 1;
        int y2 = s1;
        int y1 = s3*y2;
        int y0 = s2*y1;
        in0_type tmp[{ILP}];
        for (int i=0; i<(s1-1)/{asize*ILP}+1; i++)
        {{
            int _b3 = b3 * {bsize*ILP} + t3*{ILP};
            if (_b3 < s3) {{
                #pragma unroll
                for (int j=0; j<{ILP}; j++) {{
                    vload<sizeof(in0_type)*{ILP}>(
                        tmp,
                        &x[b0*x0+(t1*{ILP}+j+i*{asize*ILP})*x1+b2*x2+_b3*x3]
                    );
                    #pragma unroll
                    for (int k=0; k<{ILP}; k++)
                        t[(t1*{ILP}+j)*{bsize*ILP+1}+t3*{ILP}+k] = tmp[k];
                    
                }}
            }}
            __syncthreads();
            int t3_ = threadIdx.x % {asize};
            int t1_ = threadIdx.x / {asize};
            _b3 = b3 * {bsize*ILP} + t1_*{ILP};
            if (_b3 < s3) {{
                #pragma unroll
                for (int j=0; j<{ILP}; j++) {{
                    #pragma unroll
                    for (int k=0; k<{ILP}; k++) {{
                        tmp[k] =
                            t[(t3*{ILP}+k)*{bsize*ILP+1}+t1_*{ILP}+j];
                    }}
                    vload<sizeof(in0_type)*{ILP}>(
                        &y[b0*y0+b2*y1+(_b3+j)*y2+((t3*{ILP})+i*{asize*ILP})*y3],
                        tmp
                    );
                }}
            }}
            __syncthreads();
        }}
    }}
    int s0, s1, s2, s3;
    in0->shape.unpack(s0, s1, s2, s3);
    kernel<<<{{(s3-1)/{bsize*ILP}+1, s2, s0 }}, {bsize*asize}>>>
        (in0_p, out0_p, s0, s1, s2, s3);
    """)

def transpose0231_2(x):
    s0, s1, s2, s3 = x.shape
    asize = 16
    bsize = 8
    ILP = 2
    return jt.code([s0, s2, s3, s1], x.dtype, [x],
    cuda_header="#include <type/fp16_compute.h>\n#include <cassert>",
    cuda_src=f"""
    __global__ __launch_bounds__({asize*bsize}) void kernel(in0_type* __restrict__ x, in0_type* __restrict__ y, int s0, int s1, int s2, int s3) {{
        __shared__ in0_type t[{asize*ILP}*{bsize*ILP+1}];
        int t3 = threadIdx.x % {bsize};
        int t1 = threadIdx.x / {bsize};
        int b3 = blockIdx.x;
        int b1 = blockIdx.y;
        int b2 = 0;
        int b0 = blockIdx.z;
        int x3 = 1;
        int x2 = s3;
        int x1 = s2*x2;
        int x0 = s1*x1;
        int y3 = 1;
        int y2 = s1;
        int y1 = s3*y2;
        int y0 = s2*y1;
        in0_type tmp[{ILP}];
        {{
            int _b3 = b3 * {bsize*ILP} + t3*{ILP};
            if (_b3 < s3) {{
                #pragma unroll
                for (int j=0; j<{ILP}; j++) {{
                    if (t1*{ILP}+j+b1*{asize*ILP} >= s1)
                        continue;
                    vload<sizeof(in0_type)*{ILP}>(
                        tmp,
                        &x[b0*x0+(t1*{ILP}+j+b1*{asize*ILP})*x1+b2*x2+_b3*x3]
                    );
                    #pragma unroll
                    for (int k=0; k<{ILP}; k++)
                        t[(t1*{ILP}+j)*{bsize*ILP+1}+t3*{ILP}+k] = tmp[k];
                    
                }}
            }}
            __syncthreads();
            int t3_ = threadIdx.x % {asize};
            int t1_ = threadIdx.x / {asize};
            _b3 = b3 * {bsize*ILP} + t1_*{ILP};
            int yy3 = (t3_*{ILP})+b1*{asize*ILP};
            if (_b3 < s3 && yy3 < s1) {{
                #pragma unroll
                for (int j=0; j<{ILP}; j++) {{
                    #pragma unroll
                    for (int k=0; k<{ILP}; k++) {{
                        tmp[k] =
                            t[(t3_*{ILP}+k)*{bsize*ILP+1}+t1_*{ILP}+j];
                    }}
                    vload<sizeof(in0_type)*{ILP}>(
                        &y[b0*y0+b2*y1+(_b3+j)*y2+yy3*y3],
                        tmp
                    );
                    // printf("%d %d %d %d %d\\n", b0*y0+b2*y1+(_b3+j)*y2+yy3*y3,
                    //    b0, b2, (_b3+j), yy3);
                }}
            }}
            __syncthreads();
        }}
    }}
    int s0, s1, s2, s3;
    in0->shape.unpack(s0, s1, s2, s3);
    kernel<<<{{(s3-1)/{bsize*ILP}+1, (s1-1)/{asize*ILP}+1, s0 }}, {bsize*asize}>>>
        (in0_p, out0_p, s0, s1, s2, s3);
    """)

def check_share():
    return
    a = jt.rand((30, 32, 4, 2000)).float32()
    jt.code(a.shape, a.dtype, [a],
    cuda_header="#include <type/fp16_compute.h>\n#include <cassert>",
    cuda_src="""
    __global__ void kernel(in0_type* __restrict__ a, in0_type* __restrict__ b) {
        __shared__ float x[32*33];
        for (int i=0; i<3; i++) {
        ((float2*)&x[i])[0] = ((float2*)&a[i])[0];
        ((float2*)&b[i])[0] = ((float2*)&x[i+1])[0];
        }
    }
    kernel<<<1024,16*16>>>(in0_p, out0_p);
    """).sync()
    jt.sync_all(True)
    # print(a[0]+1)
    print("pass test")

class TestFP16(unittest.TestCase):
    def test_array(self):
        a = np.array([1,2,3], dtype="float16")
        b = jt.array(a)
        np.testing.assert_allclose(a, b.data)

    def test_add(self):
        a = np.array([1,2,3], dtype="float16")
        b = jt.array(a)
        c = b+b
        np.testing.assert_allclose(c.data, a+a)
        d = c.sum()
        np.testing.assert_allclose(d.data, [12])
        c = c+1
        print(c)

    def test_matmul(self):
        a = jt.random((100,100)).float16()
        b = jt.random((100,100)).float16()
        c = jt.matmul(a, b)
        c.sync()

    def test_matmul_grad(self):
        a = jt.random((100,100)).float16()
        b = jt.random((100,100)).float16()
        c = jt.matmul(a, b)
        c.sync()
        da, db = jt.grad(c, [a,b])
        jt.sync_all()
        assert da.dtype == "float16"
        assert db.dtype == "float16"

    def test_array_random_auto_cast(self):
        a = jt.array([1.0,2.0])
        assert a.dtype == "float32"
        with jt.flag_scope(amp_reg=2+16):
            a = jt.array([1.0,2.0])
            assert a.dtype == "float16", a.dtype
            
        a = jt.random([10])
        assert a.dtype == "float32"
        with jt.flag_scope(amp_reg=2+16):
            a = jt.random([10])
            assert a.dtype == "float16", a.dtype

    def test_conv(self):
        a = jt.random((3,4,5,5)).float16()
        b = jt.random((4,4,3,3)).float16()
        c = jt.nn.conv(a, b)
        c.sync()

    def test_max(self):
        a = jt.random((100,)).float16()
        b = jt.random((100,)).float16()
        c = a.maximum(b)
        c.sync()

    def test_reduce_dtype_infer(self):
        with jt.flag_scope(amp_reg=1):
            a = jt.random((3,4,5,5)).float16()
            b = a.sum()
            b.sync()
            assert b.dtype == "float32"
        with jt.flag_scope(amp_reg=2):
            a = jt.random((3,4,5,5)).float16()
            b = a.sum()
            b.sync()
            assert b.dtype == "float32"
        with jt.flag_scope(amp_reg=0):
            a = jt.random((3,4,5,5)).float16()
            b = a.sum()
            b.sync()
            assert b.dtype == "float32"
        with jt.flag_scope(amp_reg=2+4):
            a = jt.random((3,4,5,5)).float16()
            b = a.sum()
            b.sync()
            assert b.dtype == "float16", b.dtype

    def test_white_dtype_infer(self):
        with jt.flag_scope(amp_reg=1):
            a = jt.random((3,4,5,5)).float16()
            b = a**a
            b.sync()
            assert b.dtype == "float32"
        with jt.flag_scope(amp_reg=2):
            a = jt.random((3,4,5,5)).float16()
            b = a**a
            b.sync()
            assert b.dtype == "float32"
        with jt.flag_scope(amp_reg=0):
            a = jt.random((3,4,5,5)).float16()
            b = a**a
            b.sync()
            assert b.dtype == "float32"
        with jt.flag_scope(amp_reg=2+8):
            a = jt.random((3,4,5,5)).float16()
            b = a**a
            b.sync()
            assert b.dtype == "float16", b.dtype

    def test_module_half(self):
        a = jt.nn.Linear(10,10)
        assert a.weight.dtype == "float32"
        a.half()
        assert a.weight.dtype == "float16"



@unittest.skipIf(not jt.compiler.has_cuda, "No CUDA found")
class TestFP16CUDA(TestFP16):
    def setUp(self):
        jt.flags.use_cuda = 1
    def tearDown(self):
        jt.flags.use_cuda = 0

    def test_softmax(self):
        a = jt.rand((120, 2000, 2000)).float16()
        # a = jt.rand((1, 2000, 2000)).float32()
        jt.sync_all()
        with jt.profile_scope(10, 100):
            a.log_softmax(-1).sync()

    def test_transpose(self):
        check_share()
        # return
        a = jt.rand((30, 32, 4, 2000)).float32()
        # a = jt.rand((1, 1024, 1, 2000)).float32()
        diff = transpose0231(a).data != a.transpose((0,2,3,1)).data
        print(np.where(diff))
        # return
        jt.sync_all()
        # with jt.profile_scope(100, 11000):
        with jt.profile_scope(100, 11000):
            # a.log_softmax(-1).sync()
            transpose0231(a).sync()

            a.transpose((0,2,3,1)).sync()
            # a.transpose((0,2,1,3)).sync()
            a.fuse_transpose((0,2,1,3)).sync()
            (a+1).sync()
        jt.sync_all(True)
        diff = transpose0231(a).data != a.transpose((0,2,3,1)).data
        print(np.where(diff))
        np.testing.assert_allclose(transpose0231(a).data, a.transpose((0,2,3,1)).data)

    def test_transpose2(self):
        # check_share()
        # return
        # a = jt.rand((30, 32, 4, 2000)).float32()
        # a = jt.rand((1, 10000, 1, 2000)).float32()
        a = jt.rand((1, 10000, 1, 2048)).float32()
        print("transpose")
        transpose0231_2(a).sync()
        print("add")
        (a+1).sync()
        return
        # a = jt.arange(32*16).reshape((1, 32, 1, 16))
        diff = transpose0231_2(a).data != a.transpose((0,2,3,1)).data
        print(np.where(diff))
        # return
        jt.sync_all()
        # with jt.profile_scope(100, 11000):
        with jt.profile_scope(100, 1100):
            # a.log_softmax(-1).sync()
            transpose0231_2(a).sync()

            a.transpose((0,2,3,1)).sync()
            # a.transpose((0,2,1,3)).sync()
            a.fuse_transpose((0,2,1,3)).sync()
            (a+1).sync()
        jt.sync_all(True)
        diff = transpose0231_2(a).data != a.transpose((0,2,3,1)).data
        print(np.where(diff))
        np.testing.assert_allclose(transpose0231_2(a).data, a.transpose((0,2,3,1)).data)
        
if __name__ == "__main__":
    unittest.main()


# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: Dun Liang <randonlang@gmail.com>. 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import unittest
import jittor as jt
import numpy as np


class TestFuser(unittest.TestCase):
    def test_wrong_fuse(self):
        a = jt.array([1])
        b = jt.random([10,])
        c = (a * b).sum() + (a + 1)
        print(c)

    def test_wrong_fuse2(self):
        a = jt.array([1])
        b = jt.random([10,])
        c = jt.random([100,])
        bb = a*b
        cc = a*c
        jt.sync([bb,cc])
        np.testing.assert_allclose(b.data, bb.data)
        np.testing.assert_allclose(c.data, cc.data)




if __name__ == "__main__":
    unittest.main()


# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: Dun Liang <randonlang@gmail.com>. 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import unittest
import jittor as jt
import numpy as np
import os

class TestProfiler(unittest.TestCase):
    def test_profiler(self):
        a = jt.rand(1000,1000)
        b = jt.rand(1000,1000)
        jt.sync_all()
        with jt.profile_scope(10, 100, profiler_record_peek=1) as rep:
            jt.matmul(a, b).sync()
        x = float(rep[-1][4])
        y = float(rep[-2][4])
        assert abs(x-y)/x < 1e-3

if __name__ == "__main__":
    unittest.main()


# ***************************************************************
# Copyright (c) 2021 Jittor.
# All Rights Reserved. 
# Maintainers:
#     Dun Liang <randonlang@gmail.com>. 
# 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import jittor as jt
import os
import ctypes

nvtx_lib_path = os.path.join(jt.compiler.cuda_lib, "libnvToolsExt.so")
nvtx_lib = ctypes.CDLL(nvtx_lib_path, jt.compiler.dlopen_flags)

nvtxRangePushA = nvtx_lib.nvtxRangePushA
nvtxRangePop = nvtx_lib.nvtxRangePop

class nvtx_scope:
    '''
    Add a mark in nvprof timeline

    Example::

        from jittor.utils.nvtx import nvtx_scope
        with nvtx_scope("model"):
            ...

    '''
    def __init__(self, name):
        self.name = bytes(name, 'utf8')

    def __enter__(self):
        nvtxRangePushA(self.name)

    def __exit__(self, *exc):
        nvtxRangePop()

    def __call__(self, func):
        def inner(*args, **kw):
            with self:
                ret = func(*args, **kw)
            return ret
        return inner



# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: 
#     Zheng-Ning Liu <lzhengning@gmail.com> 
# 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import unittest
import jittor as jt
import jittor.nn as nn
import numpy as np


skip_this_test = False

try:
    jt.dirty_fix_pytorch_runtime_error()
    import torch
    import torch.nn as tnn
except:
    torch = None
    tnn = None
    skip_this_test = True


def check_equal(t_rnn, j_rnn, input, h0, c0):
    j_rnn.load_state_dict(t_rnn.state_dict())

    t_output, (th, tc) = t_rnn(torch.from_numpy(input), 
                              (torch.from_numpy(h0), torch.from_numpy(c0)))
    j_output, (jh, jc) = j_rnn(jt.float32(input), 
                              (jt.float32(h0), jt.float32(c0)))

    assert np.allclose(t_output.detach().numpy(), j_output.data, rtol=1e-03, atol=1e-06)
    assert np.allclose(th.detach().numpy(), jh.data, rtol=1e-03, atol=1e-06)
    assert np.allclose(tc.detach().numpy(), jc.data, rtol=1e-03, atol=1e-06)


@unittest.skipIf(skip_this_test, "No Torch found")
class TestLSTM(unittest.TestCase):
    def test_lstm_cell(self):
        np_h0 = torch.randn(3, 20).numpy()
        np_c0 = torch.randn(3, 20).numpy()

        t_rnn = tnn.LSTMCell(10, 20) 
        input = torch.randn(2, 3, 10)
        h0 = torch.from_numpy(np_h0)
        c0 = torch.from_numpy(np_c0)
        t_output = []
        for i in range(input.size()[0]):
            h0, c0 = t_rnn(input[i], (h0, c0))
            t_output.append(h0)
        t_output = torch.stack(t_output, dim=0)

        j_rnn = nn.LSTMCell(10, 20)
        j_rnn.load_state_dict(t_rnn.state_dict())

        input = jt.float32(input.numpy())
        h0 = jt.float32(np_h0)
        c0 = jt.float32(np_c0)
        j_output = []
        for i in range(input.size()[0]):
            h0, c0 = j_rnn(input[i], (h0, c0))
            j_output.append(h0)
        j_output = jt.stack(j_output, dim=0)

        t_output = t_output.detach().numpy()
        j_output = j_output.data
        assert np.allclose(t_output, j_output, rtol=1e-03, atol=1e-06)

    def test_lstm(self):
        h0 = np.random.rand(1, 2, 20).astype(np.float32)
        c0 = np.random.rand(1, 2, 20).astype(np.float32)
        input = np.random.rand(5, 2, 10).astype(np.float32)

        t_rnn = tnn.LSTM(10, 20) 
        j_rnn = nn.LSTM(10, 20)
        check_equal(t_rnn, j_rnn, input, h0, c0)

        proj_size = 13
        h0 = np.random.rand(1, 2, proj_size).astype(np.float32)
        c0 = np.random.rand(1, 2, 20).astype(np.float32)
        input = np.random.rand(5, 2, 10).astype(np.float32)
        t_rnn = tnn.LSTM(10, 20, proj_size=proj_size) 
        j_rnn = nn.LSTM(10, 20, proj_size=proj_size)
        check_equal(t_rnn, j_rnn, input, h0, c0)

        h0 = np.random.rand(2, 4, 20).astype(np.float32)
        c0 = np.random.rand(2, 4, 20).astype(np.float32)
        input = np.random.rand(5, 4, 10).astype(np.float32)

        t_rnn = tnn.LSTM(10, 20, num_layers=2) 
        j_rnn = nn.LSTM(10, 20, num_layers=2)
        check_equal(t_rnn, j_rnn, input, h0, c0)

        h0 = np.random.rand(2, 4, proj_size).astype(np.float32)
        c0 = np.random.rand(2, 4, 20).astype(np.float32)
        input = np.random.rand(5, 4, 10).astype(np.float32)

        t_rnn = tnn.LSTM(10, 20, num_layers=2, proj_size=proj_size) 
        j_rnn = nn.LSTM(10, 20, num_layers=2, proj_size=proj_size)
        check_equal(t_rnn, j_rnn, input, h0, c0)


if __name__ == "__main__":
    unittest.main()


import jittor as jt
from jittor import nn
import numpy as np
# import pylab as pl

# 隐空间向量长度
latent_dim = 100
# 类别数量
n_classes = 10
# 图片大小
img_size = 32
# 图片通道数量
channels = 1
# 图片张量的形状
img_shape = (channels, img_size, img_size)

class Generator(nn.Module):
    def __init__(self):
        super(Generator, self).__init__()
        self.label_emb = nn.Embedding(n_classes, n_classes)

        def block(in_feat, out_feat, normalize=True):
            layers = [nn.Linear(in_feat, out_feat)]
            if normalize:
                layers.append(nn.BatchNorm1d(out_feat, 0.8))
            layers.append(nn.LeakyReLU(0.2))
            return layers
        self.model = nn.Sequential(
            *block((latent_dim + n_classes), 128, normalize=False), 
            *block(128, 256), 
            *block(256, 512), 
            *block(512, 1024), 
            nn.Linear(1024, int(np.prod(img_shape))), 
            nn.Tanh())

    def execute(self, noise, labels):
        gen_input = jt.contrib.concat((self.label_emb(labels), noise), dim=1)
        img = self.model(gen_input)
        img = img.view((img.shape[0], *img_shape))
        return img

class Discriminator(nn.Module):
    def __init__(self):
        super(Discriminator, self).__init__()
        self.label_embedding = nn.Embedding(n_classes, n_classes)
        self.model = nn.Sequential(
            nn.Linear((n_classes + int(np.prod(img_shape))), 512), 
            nn.LeakyReLU(0.2), 
            nn.Linear(512, 512), 
            nn.Dropout(0.4), 
            nn.LeakyReLU(0.2), 
            nn.Linear(512, 512), 
            nn.Dropout(0.4), 
            nn.LeakyReLU(0.2), 
            nn.Linear(512, 1))

    def execute(self, img, labels):
        d_in = jt.contrib.concat((img.view((img.shape[0], (- 1))), self.label_embedding(labels)), dim=1)
        validity = self.model(d_in)
        return validity


# 定义模型
generator = Generator()
discriminator = Discriminator()
generator.eval()
discriminator.eval()

# 加载参数
generator.load('https://cg.cs.tsinghua.edu.cn/jittor/assets/build/generator_last.pkl')
discriminator.load('https://cg.cs.tsinghua.edu.cn/jittor/assets/build/discriminator_last.pkl')



def gen_img(number):
    print(number, type(number))
    n_row = len(number)
    z = jt.array(np.random.normal(0, 1, (n_row, latent_dim))).float32().stop_grad()
    labels = jt.array(np.array([int(number[num]) for num in range(n_row)])).float32().stop_grad()
    gen_imgs = generator(z,labels)
    gen_imgs = gen_imgs.transpose((1,2,0,3)).reshape(gen_imgs.shape[2], -1)
    gen_imgs = gen_imgs[:,:,None].broadcast(gen_imgs.shape+(3,)) # .uint8()
    gen_imgs = (gen_imgs - gen_imgs.min()) / (gen_imgs.max() - gen_imgs.min()) * 255
    gen_imgs = gen_imgs.uint8()
    # print(gen_imgs.shape, gen_imgs.max(), gen_imgs.min())
    return gen_imgs.numpy()
    # gen_imgs = gen_imgs.data.transpose((1,2,0,3))[0].reshape((gen_imgs.shape[2], -1))
    # print(gen_imgs.shape)
    return gen_imgs[:,:,None]

from PIL import Image
import pywebio as pw
# 定义一串数字
number = "201962517"
# gen_img(number)
Image.fromarray(gen_img(number))
# pl.imshow()
# pl.show()
# print("done")


def web_server():
    pw.pin.put_input("number", label="输入用于生成的数字(由计图框架支持)：")
    pw.output.put_buttons(['Gen image'], 
        lambda _: pw.output.put_image(Image.fromarray(gen_img(pw.pin.pin.number))))

pw.start_server(web_server, port=8123)


error_msg = """Jittor only supports Linux and macOS currently.
For other OS, use Jittor may be risky.
If you insist on installing, please set the environment variable : export FORCE_INSTALL=1
We strongly recommend docker installation:

# CPU only (Linux)
>>> docker run -it --network host jittor/jittor
# CPU and CUDA (Linux)
>>> docker run -it --network host jittor/jittor-cuda
# CPU only (Mac and Windows)
>>> docker run -it -p 8888:8888 jittor/jittor

Reference:
1. Windows/Mac/Linux install Jittor in Docker: https://cg.cs.tsinghua.edu.cn/jittor/tutorial/2020-5-15-00-00-docker/
"""
from warnings import warn
import os
import platform

if not platform.system() in ['Linux', 'Darwin']:
    assert os.environ.get("FORCE_INSTALL", '0') != '1', error_msg

import setuptools
from setuptools import setup, find_packages
import os
import sys

path = os.path.dirname(__file__)
with open(os.path.join(path, "README.md"), "r", encoding='utf8') as fh:
    long_description = fh.read()

with open(os.path.join(path, "python/jittor/__init__.py"), "r", encoding='utf8') as fh:
    for line in fh:
        if line.startswith('__version__'):
            version = line.split("'")[1]
            break
    else:
        raise RuntimeError("Unable to find version string.")

version_require = (3,7)
if os.name == 'nt':
    version_require = (3,8)
if sys.version_info < version_require:
    raise RuntimeError("Python version not match, require %s, current %s"
        %(version_require, sys.version_info))

setuptools.setup(
    name='jittor',  
    version=version,
    # scripts=[],
    author="Jittor Group",
    author_email="ran.donglang@gmail.com",
    description="a Just-in-time(JIT) deep learning framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://jittor.org",
    # packages=setuptools.find_packages(),
    python_requires='>=3.7',

    packages=["jittor", "jittor.test", "jittor.models", "jittor.utils", "jittor_utils"],
    package_dir={'': 'python'},
    package_data={'': ['*', '*/*', '*/*/*','*/*/*/*','*/*/*/*/*','*/*/*/*/*/*']},
    # include_package_data=True,
    install_requires=[
        "numpy",
        "tqdm",
        "pillow",
        "astunparse",
        'pywin32 >= 1.0 ; platform_system=="Windows"'
    ],
 )

# upload to pip:
# rm -rf dist && python3.7 ./setup.py sdist && python3.7 -m twine upload dist/*



# ***************************************************************
# Copyright (c) 2021 Jittor. All Rights Reserved. 
# Maintainers: Dun Liang <randonlang@gmail.com>. 
# This file is subject to the terms and conditions defined in
# file 'LICENSE.txt', which is part of this source code package.
# ***************************************************************
import unittest
import jittor as jt
import numpy as np
from .test_core import expect_error
from .test_grad import ngrad
from itertools import permutations
from jittor.test.test_cuda import test_cuda

def gen_data(shape):
    num = np.multiply.reduce(shape)
    a = np.arange(0, num)
    return a.reshape(shape)

class TestTransposeOp(unittest.TestCase):
    def test_with_np(self):
        def check(a):
            perms = list(permutations(range(a.ndim))) + [None]
            for perm in perms:
                if perm:
                    x = np.transpose(a, perm)
                    y = jt.transpose(a, perm).data
                else:
                    x = np.transpose(a)
                    y = jt.transpose(a).data
                self.assertEqual(x.shape, y.shape)
                assert (x==y).all(), f"\n{x}\n{y}"
                
        # ia = [gen_data([2,3,4,5]), gen_data([5,3])]
        ia = [gen_data([2,2,2]), gen_data([2,3,4,5]), gen_data([5,3])]
        for a in ia: check(a)
        
    def test_grad(self):
        def check(a):
            perms = list(permutations(range(a.ndim))) + [None]
            for perm in perms:
                x = jt.array(a).float()
                if perm:
                    y = x.transpose(perm)
                else:
                    y = x.transpose()
                dx = jt.grad(y*y, x).data
                self.assertEqual(dx.shape, a.shape)
                assert (dx==a*2).all(), f"\n{dx}\n{a}\n{perm}"
        ia = [gen_data([2,2,2]), gen_data([2,3,4,5]), gen_data([5,3])]
        for a in ia: check(a)
        
    def test_matmul_grad(self):
        np.random.seed(0)
        for i in range(10):
            a = np.random.rand(2,3).astype("float32")
            b = np.random.rand(3,4).astype("float32")
            out, (da, db) = ngrad(lambda vars: np.matmul(vars[0],vars[1]).sum(), [a,b], 1e-1)
            ja = jt.array(a)
            jb = jt.array(b)
            jc = ja.matmul(jb)
            jda, jdb = jt.grad(jc, [ja,jb])
            assert ((da-jda.data)<1e-5).all(), (da, jda.data, da-jda.data)
            assert ((db-jdb.data)<1e-5).all(), (db-jdb.data)

    def test_permute(self):
        a = jt.ones([2,3,4])
        assert a.permute().shape == [4,3,2]
        assert a.permute(0,2,1).shape == [2,4,3]

    def test_transpose_3d2i(self):
        a = jt.ones([2,3,4])
        assert a.transpose(0,1).shape == (3,2,4)

    @unittest.skipIf(not jt.compiler.has_cuda, "No CUDA found")
    @jt.flag_scope(use_cuda=1)
    def test_cutt(self):
        a = jt.rand((10,2)) > 0.5
        b = a.transpose()
        assert (a.data.transpose() == b.data).all()

        a = jt.zeros((1,1))
        b = a.transpose((1,0))
        b.sync()


class TestFuseTransposeOp(unittest.TestCase):

    def test_fuse_transpose1(self):
        with jt.profile_scope() as rep:
            a = jt.rand((10,11,12))
            b = a.fuse_transpose((1,2,0))+1
            np.testing.assert_allclose(
                a.data.transpose((1,2,0))+1,
                b.data
            )
        assert len(rep) == 3

    def test_fuse_transpose2(self):
        with jt.profile_scope() as rep:
            a = jt.rand((10,11,12))
            b = (a+1).fuse_transpose((1,2,0))
            np.testing.assert_allclose(
                a.data.transpose((1,2,0))+1,
                b.data
            )
        assert len(rep) == 3

    def test_fuse_transpose3(self):
        with jt.profile_scope() as rep:
            a = jt.rand((10,11,12))
            c = jt.rand((11,12,10))
            b = a.fuse_transpose((1,2,0))+c
            np.testing.assert_allclose(
                a.data.transpose((1,2,0))+c.data,
                b.data
            )
        assert len(rep) == 3

    def test_fuse_transpose4(self):
        with jt.profile_scope() as rep:
            a = jt.rand((10,11,12))
            c = jt.rand((10,11,12))
            b = (a+c).fuse_transpose((1,2,0))
            np.testing.assert_allclose(
                (a.data+c.data).transpose((1,2,0)),
                b.data
            )
        assert len(rep) == 3

    def test_fuse_transpose5(self):
        with jt.profile_scope() as rep:
            a = jt.rand((10,11,6,7))
            c = jt.rand((10,11,6,7))
            b = (a+c).fuse_transpose((1,0,2,3))
            np.testing.assert_allclose(
                (a.data+c.data).transpose((1,0,2,3)),
                b.data
            )
        assert len(rep) == 3


@unittest.skipIf(not jt.compiler.has_cuda, "No CUDA found")
class TestFuseTransposeCudaOp(TestFuseTransposeOp):
    def setUp(self):
        jt.flags.use_cuda = 1
    def tearDown(self):
        jt.flags.use_cuda = 0

if __name__ == "__main__":
    unittest.main()